from playwright.sync_api import TimeoutError as PwTimeoutError
from playwright.sync_api import sync_playwright

from inni.modules.base import BaseModule


class Module(BaseModule):
    login_template_keys = ()
    logout_template_keys = ()

    def setUp(self):
        self.headless = self.config.get("headless", False)
        self.url = self.config["url"]
        self.username = self.config["username"]
        self.password = self.config["password"]

    def visit(self, login: bool):
        in_or_out = "in" if login else "out"
        with sync_playwright() as pw, self.out.status(
            f"Clocking {in_or_out}"
        ) as status:
            browser = pw.firefox.launch(headless=self.headless)
            context = browser.new_context(viewport={"width": 1800, "height": 900})
            page = context.new_page()
            status.update("Opening webpage")
            page.goto(self.url, timeout=10000000, wait_until="domcontentloaded")

            # Login
            status.update("Filling in username")
            page.get_by_label("Enter phone number or email").click()
            page.get_by_label("Enter phone number or email").fill(
                self.config["username"]
            )
            # Wait a second for bitrix JS to enable the button
            page.wait_for_timeout(1000)
            page.get_by_label("Enter phone number or email").press("Enter")
            while True:
                page.get_by_role("button", name="Next").click()
                status.update("Filling in password")
                try:
                    page.get_by_label("Enter your password to").click(timeout=1000)
                    break
                except PwTimeoutError:
                    status.update("Could not find password field, retrying username.")
            status.update("Filling in password")
            page.get_by_label("Enter your password to").fill(self.config["password"])
            # Wait a second for bitrix JS to enable the button
            page.wait_for_timeout(1000)
            page.get_by_label("Enter your password to").press("Enter")
            page.get_by_role("button", name="Next").click()

            self.out.print("[green]✅Logged into bitrix")
            # Close the popup
            status.update("Closing popup")
            page.locator(".popup-window-close-icon").click()

            # Login/Logout
            status.update(f"Clocking {in_or_out}")
            page.locator("span#timeman-block").click()
            if login:
                page.get_by_role("button", name="Clock In").click()
            else:
                page.get_by_role("button", name="Clock Out").click()

            # Ensure bitrix registered the event
            status.update(
                f"Clocked {in_or_out}. Waiting for bitrix to register the event."
            )
            if login:
                page.get_by_role("button", name="Clock Out").wait_for()
            else:
                page.get_by_role("button", name="Clock In").wait_for()
            page.wait_for_timeout(1000)

            context.close()
            browser.close()
        self.out.print(f"[green]✅ Clocked {in_or_out}")

    def login(self, responses):
        self.visit(True)

    def logout(self, responses):
        self.visit(False)



import sys
from importlib import import_module
from pathlib import Path
from typing import Any, Optional

import click
from rich.console import Console

from inni.config import default_config, read_config
from inni.loader import load_module

console = Console()
error_console = Console(stderr=True, style="red bold")


@click.group()
@click.option(
    "-c", "--config", type=click.Path(exists=True, dir_okay=False, readable=True)
)
@click.option(
    "-v", "--vars-file", type=click.Path(exists=True, dir_okay=False, readable=True)
)
@click.pass_context
def inni(ctx: click.Context, config: Optional[str], vars_file: Optional[str]):
    ctx.ensure_object(dict)
    if config is None:
        config = str(default_config())
    ctx.obj["config"] = read_config(config)
    if vars_file is not None:
        vars_module: Path = Path(vars_file)
    else:
        vars_module: Path = default_config().parent / "vars.py"

    if vars_module.exists():
        module_dir = vars_module.parent
        sys.path.append(str(module_dir))
        ctx.obj["vars"] = import_module(vars_module.name.removesuffix(".py"))
        sys.path.remove(str(module_dir))


def _login_out(ctx: click.Context, login: bool):
    config = ctx.obj["config"]
    module_names = config["inni"]["login" if login else "logout"]
    prompts = config["inni"]["prompts"]
    modules = {}

    with console.status("") as status:
        for module_name in module_names:
            status.update(f"[green]Initializing {module_name}")
            try:
                module = load_module(module_name)
                module_config = config["modules"].get(module_name, {})
                modules[module_name] = module(module_config, console, error_console)
            except ModuleNotFoundError as err:
                status.stop()
                error_console.print(err.msg)
                sys.exit(0)
    console.print("[green]✅ Initialized Modules")
    console.print("")

    variables_to_prompt = set()
    for module in modules.values():
        variables_to_prompt.update(
            module.template_variables()["login" if login else "logout"]
        )
    variables_to_prompt -= {"vars"}

    responses: dict[str, Any] = {
        "vars": ctx.obj.get("vars", {}),
    }
    for var in variables_to_prompt:
        prompt = prompts.get(var, f"{var}:").strip() + " "
        responses[var] = console.input("[blue]" + prompt)

    console.print(f"\n[green bold]Logging {'In' if login else 'Out'}\n")
    for name, module in modules.items():
        console.rule(f"[bold cyan]Module: {name}", align="left")
        try:
            if login:
                module.login(responses)
            else:
                module.logout(responses)
        except Exception:
            error_console.print(f"[red bold]Fatal error occured running module {name}")
            error_console.print_exception()


@inni.command()
@click.pass_context
def login(ctx: click.Context):
    return _login_out(ctx, True)


@inni.command()
@click.pass_context
def logout(ctx: click.Context):
    return _login_out(ctx, False)



from importlib import import_module
from typing import Type

from inni.modules.base import BaseModule


def load_module(name: str) -> Type[BaseModule]:
    try:
        mod = import_module("inni.modules." + name)
    except ModuleNotFoundError:
        mod = import_module(name)

    return mod.Module



from inni.modules.base import BaseModule
from inni.template import render_template


class Module(BaseModule):
    login_template_keys = ("login_template",)
    logout_template_keys = ("logout_template",)

    def login(self, responses):
        x = render_template(self.config["login_template"], **responses)
        print("Rendered template:", x)

    def logout(self, responses):
        x = render_template(self.config["logout_template"], **responses)
        print("Rendered template:", x)



import os
import tomllib
from pathlib import Path


def default_config() -> Path:
    conf_file = Path.home() / ".config" / "inni" / "config.toml"
    if xdg_dir := os.environ.get("XDG_CONFIG_HOME"):
        path = Path(xdg_dir)
        xdg_file = path / "inni" / "config.toml"
        if xdg_file.exists():
            return xdg_file
        elif conf_file.exists():
            return conf_file
        else:
            return xdg_file
    return conf_file


def read_config(path: str):
    with open(path, "rb") as f:
        content = tomllib.load(f)

    defaults = {
        "login": [],
        "logout": [],
        "prompts": {},
    }

    if "inni" not in content:
        content["inni"] = defaults

    for key, value in defaults.items():
        if key not in content["inni"]:
            content[key] = value

    return content



from jinja2 import Environment, FunctionLoader, meta

ENV = Environment(loader=FunctionLoader(lambda i: i))


def template_to_vars(source: str) -> set[str]:
    """
    Returns the variables present in the template
    """
    return meta.find_undeclared_variables(ENV.parse(source))


def render_template(source: str, **kwargs):
    template = ENV.get_template(source)
    return template.render(**kwargs)
